\name{SparseM.hb}
\alias{SparseM.hb}
\alias{read.matrix.hb}
\alias{write.matrix.hb}
\alias{model.matrix,ANY-method}
\alias{model.matrix,matrix.csc.hb-method}
\alias{model.matrix,matrix.ssc.hb-method}
\alias{model.matrix.matrix.ssc.hb}
\alias{model.response,ANY-method}
\alias{model.response,matrix.csc.hb-method}
\alias{model.response,matrix.ssc.hb-method}
\alias{model.response}
\alias{model.matrix}
\title{Harwell-Boeing Format Sparse Matrices}
\description{
Read, write and extract components of data in Harwell-Boeing  
sparse matrix format.
}
\usage{
read.matrix.hb(filename)
write.matrix.hb(filename = "hb.out", X, title, key, mxtype, rhs = NULL, 
guess = FALSE,  xsol = FALSE, ptrfmt = "(16I5)", indfmt = "(16I5)",
valfmt = "(1P,5D16.9)", rhsfmt = "(1P,5D16.9)")
model.matrix(object, \dots)
model.response(data,type)
}
\arguments{
  \item{filename}{file name to read from or write to }
  \item{data, object}{an object of either \code{matrix.csc.hb} or \code{matrix.ssc.hb} 
class}
  \item{type}{One of `"any"', `"numeric"', `"double"'. Using the either of
          latter two coerces the result to have storage mode
          `"double"'}
  \item{X}{coefficient matrix stored in csc (for unsymmetric and rectangular
        matrix) or ssc (for symmetric matrix) format}
  \item{title}{72-character title for the matrix}
  \item{key}{8-character identifier for the matrix}
  \item{mxtype}{3-character identifier for type of the coefficient matrix;\cr
        First character: currently only "R" for real matrix is supported;
        second character: "S" for symmetric, "U" for unsymmetric" 
		and "R" for rectangular matrix;
        third character: currently only "A" for assembled matrix is supported }
  \item{rhs}{vector or matrix of right-hand-side(s) including starting guesses
        and solution vectors if present stored in full storage mode}
  \item{guess}{logical flag for the presence of initial guess of the solutions;
        if TRUE, the values of initial guess are appended to the end of
        \code{rhs}}
  \item{xsol}{logical flag for the presence of exact solutions; if TRUE, the
        values of the exact solutions are appended to the end of \code{rhs}}
  \item{ptrfmt}{printing format for the column pointers}
  \item{indfmt}{printing format for the row indices}
  \item{valfmt}{printing format for the values}
  \item{rhsfmt}{printing format for the right-hand-sides}
  \item{\dots}{additional arguments}
}
\details{
Sparse coefficient matrices in the Harwell-Boeing format are stored in 
80-column records. Each file begins with a multiple line header block 
followed by two, three or four data blocks. The header block contains 
summary information on the storage formats and storage requirements. 
The data blocks contain information of the sparse coefficient matrix and 
data for the right-hand-side of the linear system of equations, 
initial guess of the solution and the exact solutions if they exist.
The function \code{model.matrix} extracts the X matrix component.
The function \code{model.response} extracts the y vector (or matrix).
}
\value{
The function \code{read.matrix.hb} returns a list of class \code{matrix.csc.hb} or \code{matrix.ssc.hb} depending
on how the coefficient matrix is stored in the \code{file}.
  \item{ra }{ra component of the csc or ssc format of the coefficient matrix, X.}
  \item{ja }{ja component of the csc or ssc format of the coefficient matrix, X.}
  \item{ia }{ia component of the csc or ssc format of the coefficient matrix, X.}
  \item{rhs.ra }{ra component of the right-hand-side, y, if stored in csc or 
	ssc format; right-hand-side stored in dense vector or matrix otherwise.}
  \item{rhs.ja }{ja component of the right-hand-side, y, if stored in csc or 
	ssc format; a null vector otherwise.}
  \item{rhs.ia }{ia component of the right-hand-side, y, if stored in csc or 
	ssc format; a null vector otherwise.}
  \item{xexact}{vector of the exact solutions, b, if they exist; a null vector otherwise.}
  \item{guess}{vector of the initial guess of the solutions if they exist; 
	a null vector otherwise.}
  \item{dimension}{dimenson of the coefficient matrix, X.}
  \item{rhs.dim}{dimenson of the right-hand-side, y.}
  \item{rhs.mode}{storage mode of the right-hand-side; can be full storage or
	same format as the coefficient matrix.}

The function \code{model.matrix} returns the X matrix of class \code{matrix.csr}.
The function \code{model.response} returns the y vector (or matrix).
}
\references{ 
Duff, I.S., Grimes, R.G. and Lewis, J.G. (1992) 
User's Guide for Harwell-Boeing Sparse Matrix Collection at 
\url{http://math.nist.gov/MatrixMarket/collections/hb.html}}
\author{Pin Ng}
\seealso{
\code{slm} for sparse version of \code{lm} \cr
\code{SparseM.ops} for operators on class \code{matrix.csr} \cr
\code{SparseM.solve} for linear equation solving for class \code{matrix.csr} \cr
\code{SparseM.image} for image plotting of class \code{matrix.csr} \cr
\code{SparseM.ontology} for coercion of class \code{matrix.csr} \cr

}

\examples{
\dontrun{read.matrix.hb(system.file("HBdata","lsq.rra",package = "SparseM"))-> hb.o
class(hb.o) # -> [1] "matrix.csc.hb"
model.matrix(hb.o)->X
class(X) # -> "matrix.csr"
dim(X) # -> [1] 1850  712
y <- model.response(hb.o) # extract the rhs
length(y) # [1] 1850
}
}
\keyword{IO}

\eof
\name{SparseM.image}
\alias{image,matrix.csr-method}
\alias{image.matrix.csc-method}
\alias{image,ANY-method}
\alias{SparseM.image}
\alias{image}
\title{Image Plot for Sparse Matrices}
\description{Display the pattern of non-zero entries of 
a matrix of class \code{matrix.csr} or \code{matrix.csc}}
\usage{
image(x, \dots)
}
\arguments{
\item{x}{a matrix of class \code{matrix.csr} or \code{matrix.csc}.}
\item{\dots}{additional arguments.}
}
\details{The pattern of the non-zero entries of a sparse matrix 
is displayed.  By default nonzero entries of the matrix appear
as gray blocks and zero entries as white background.}
\references{
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R,\cr
\url{http://www.econ.uiuc.edu/~roger/research}}
\seealso{
\code{SparseM.ops}, 
\code{SparseM.solve}, 
\code{SparseM.ontology} 
}
\examples{
a <- rnorm(20*5)
A <- matrix(a,20,5)
A[row(A)>col(A)+4|row(A)<col(A)+3] <- 0
b <- rnorm(20*5)
B <- matrix(b,20,5)
B[row(A)>col(A)+2|row(A)<col(A)+2] <- 0
image(as.matrix.csr(A)\%*\%as.matrix.csr(t(B)))
}
\keyword{hplot}
\keyword{algebra}

\eof
\name{SparseM.ontology}
\alias{SparseM.ontology}
\alias{matrix.csr}
\alias{matrix.csc}
\alias{initialize,ANY-method}
\alias{initialize,matrix.csr-method}
\alias{initialize,matrix.coo-method}
\alias{coerce,vector,matrix.diag.csr-method}
\alias{coerce,matrix.csr,matrix.diag.csr-method}
\alias{coerce,vector,matrix.csr-method}
\alias{as.matrix,ANY-method}
\alias{as.matrix,matrix.csr-method}
\alias{as.matrix,csr-method}
\alias{is.matrix,csr-methods} 
\alias{as.matrix,csc-methods}
\alias{is.matrix,csc-methods} 
\alias{as.matrix,ssr-methods}
\alias{is.matrix,ssr-methods} 
\alias{as.matrix,ssc-methods}
\alias{is.matrix,ssc-methods} 
\alias{as.matrix,coo-methods}
\alias{is.matrix,coo-methods} 
\alias{as.matrix.csr,ANY-method}
\alias{as.matrix.csr,matrix.csc-method}
\alias{as.matrix.csr,matrix.ssr-method}
\alias{as.matrix.csr,matrix.ssc-method}
\alias{as.matrix.csr,matrix.coo-method}
\alias{as.matrix.csc,ANY-method}
\alias{as.matrix.csc,matrix.csr-method}
\alias{as.matrix.csc,matrix.csc-method}
\alias{as.matrix.csc,matrix.ssr-method}
\alias{as.matrix.csc,matrix.ssc-method}
\alias{as.matrix.csc,matrix.coo-method}
\alias{as.matrix.ssr,ANY-method}
\alias{as.matrix.ssr,matrix.csc-method}
\alias{as.matrix.ssr,matrix.ssr-method}
\alias{as.matrix.ssr,matrix.ssc-method}
\alias{as.matrix.ssr,matrix.coo-method}
\alias{as.matrix.ssc,ANY-method}
\alias{as.matrix.ssc,matrix.csr-method}
\alias{as.matrix.ssc,matrix.csc-method}
\alias{as.matrix.ssc,matrix.ssr-method}
\alias{as.matrix.ssc,matrix.ssc-method}
\alias{as.matrix.ssc,matrix.coo-method}
\alias{as.matrix.coo,ANY-method}
\alias{as.matrix.coo,matrix.csr-method}
\alias{as.matrix.coo,matrix.csc-method}
\alias{as.matrix.coo,matrix.ssr-method}
\alias{as.matrix.coo,matrix.ssc-method}
\alias{as.matrix.coo,matrix.coo-method}
\alias{as.matrix,matrix.csc-method}      
\alias{as.matrix,matrix.ssc-method}     
\alias{as.matrix,matrix.ssr-method}      
\alias{as.matrix,matrix.coo-method}      
\alias{matrix.ssc}            
\alias{as.matrix.ssc,matrix.csc-method} 
\alias{as.matrix.ssc.matrix.csr-method}  
\alias{as.matrix.ssc,matrix.ssr-method} 
\alias{matrix.ssr}             
\alias{as.matrix.ssr,matrix.csc-method}  
\alias{as.matrix.ssr,matrix.csr-method} 
\alias{as.matrix.ssr,matrix.ssc-method}  
\alias{is.matrix.csr}
\alias{is.matrix.csc}
\alias{is.matrix.ssr}
\alias{is.matrix.ssc}
\alias{is.matrix.coo}
\alias{is.matrix.csr}
\alias{as.matrix.csr}
\alias{as.matrix.csc}
\alias{as.matrix.ssr}
\alias{as.matrix.ssc}
\alias{as.matrix.coo}
\title{Sparse Matrix Class}
\description{
This group of functions evaluates and coerces changes in class structure.
}
\usage{
as.matrix.csr(x, nrow = 1, ncol = 1, eps = .Machine$double.eps)
as.matrix.csc(x, nrow = 1, ncol = 1, eps = .Machine$double.eps)
as.matrix.ssr(x, nrow = 1, ncol = 1, eps = .Machine$double.eps)
as.matrix.ssc(x, nrow = 1, ncol = 1, eps = .Machine$double.eps)
is.matrix.csr(x, ...)
is.matrix.csc(x, ...)
is.matrix.ssr(x, ...)
is.matrix.ssc(x, ...)
}
\arguments{
\item{x}{is a matrix, or vector object,  of either dense or sparse form}
\item{nrow}{number of rows of matrix }
\item{ncol}{number of columns of matrix }
\item{eps}{A tolerance parameter:  elements of x such that abs(x) < eps set to zero.
This argument is only relevant when coercing matrices from dense to sparse form. Defaults to
\code{eps = .Machine$double.eps} }
\item{...}{other arguments}

}
\details{
The function \code{matrix.csc} acts like \code{matrix} to coerce a vector object to
a sparse matrix object of class \code{matrix.csr}.  
This aspect of the code is in the process of conversion from S3 to S4 classes.
For the most part the S3 syntax prevails.  An exception is the code to
coerce vectors to diagonal matrix form which uses \code{as(v,"matrix.diag.csr"}.
The generic functions \code{as.matrix.xxx} coerce a matrix \code{x} into 
a matrix of storage class \code{matrix.xxx}. The argument matrix \code{x}
may be of conventional dense form, or of any of the four supported 
classes:  \code{matrix.csr, matrix.csc, matrix.ssr, matrix.ssc}.  
The generic functions \code{is.matrix.xxx} evaluate whether the  
argument is of class \code{matrix.xxx}.  The function
\code{as.matrix} transforms a matrix of any sparse class into conventional
dense form.  The primary storage class for sparse matrices is the
compressed sparse row \code{matrix.csr} class.
An \emph{n} by \emph{m} matrix \emph{A} with real elements \eqn{a_{ij}}{a_{ij}}, 
stored in \code{matrix.csr} format consists of three arrays:
 
\item \code{ra}: a real array of \emph{nnz} elements containing the non-zero
elements of \emph{A}, stored in row order. Thus, if \emph{i<j}, all elements of row \emph{i}
precede elements from row \emph{j}. The order of elements within the rows is immaterial.
 
\item \code{ja}: an integer array of \emph{nnz} elements containing the column
indices of the elements stored in \code{ra}.
 
\item \code{ia}: an integer array of \emph{n+1} elements containing pointers to
the beginning of each row in the arrays \code{ra} and \code{ja}. Thus
\code{ia[i]} indicates the position in the arrays \code{ra} and
\code{ja} where the \emph{i}th row begins. The last, \emph{(n+1)}st, element of
\code{ia} indicates where the \emph{n+1} row would start, if it existed.

The compressed sparse column class  \code{matrix.csc} is defined in
an analogous way, as are  the \code{matrix.ssr}, symmetric sparse row, and
\code{matrix.ssc}, symmetric sparse column classes.
}
\note{
\code{as.matrix.ssr} and \code{as.matrix.ssc} should ONLY be used with 
symmetric matrices.
}
\references{
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R,\cr
\url{http://www.econ.uiuc.edu/~roger/research}
}
\seealso{
\code{SparseM.hb} for handling Harwell-Boeing sparse matrices.
}
\examples{
n1 <- 10
p <- 5
a <- rnorm(n1*p)
a[abs(a)<0.5] <- 0
A <- matrix(a,n1,p)
B <- t(A)\%*\%A
A.csr <- as.matrix.csr(A)
A.csc <- as.matrix.csc(A)
B.ssr <- as.matrix.ssr(B)
B.ssc <- as.matrix.ssc(B)
is.matrix.csr(A.csr) # -> TRUE
is.matrix.csc(A.csc) # -> TRUE
is.matrix.ssr(B.ssr) # -> TRUE
is.matrix.ssc(B.ssc) # -> TRUE
as.matrix(A.csr)
as.matrix(A.csc)
as.matrix(B.ssr)
as.matrix(B.ssc)
as.matrix.csr(rep(0,9),3,3) #sparse matrix of all zeros
}
\keyword{algebra}

\eof
\name{SparseM.ops}
\alias{Ops.matrix.csr}
\alias{Ops.matrix.diag.csr}
\alias{\%*\%-methods}
\alias{\%*\%,ANY,ANY-method}
\alias{\%*\%,matrix.csr,matrix.csr-method}
\alias{\%*\%,matrix.csr,matrix-method}
\alias{\%*\%,matrix.csr,numeric-method}
\alias{\%*\%,matrix,matrix.csr-method}
\alias{\%*\%,numeric,matrix.csr-method}
\alias{\%x\%-methods}
\alias{\%x\%,ANY,ANY-method}
\alias{\%x\%,matrix.csr,matrix.csr-method}
\alias{\%x\%,matrix.csr,matrix-method}
\alias{\%x\%,matrix.csr,numeric-method}
\alias{\%x\%,matrix,matrix.csr-method}
\alias{\%x\%,numeric,matrix.csr-method}
\alias{+,matrix.csr-method}
\alias{-,matrix.csr-method}
\alias{*,matrix.csr-method}
\alias{/,matrix.csr-method}
\alias{^,matrix.csr-method}
\alias{\%\%,matrix.csr-method}
\alias{\%/\%,matrix.csr-method}
\alias{>,matrix.csr-method}
\alias{>=,matrix.csr-method}
\alias{<,matrix.csr-method}
\alias{<=,matrix.csr-method}
\alias{==,matrix.csr-method}
\alias{!=,matrix.csr-method}
\alias{&,matrix.csr-method}
\alias{|,matrix.csr-method}
\alias{norm} 
\alias{norm,ANY-method} 
\alias{norm,matrix.csr-method} 
\alias{det,ANY-method} 
\alias{det,matrix-method} 
\alias{det,matrix.csr-method} 
\alias{det,matrix.csr.chol-method} 
\alias{t,ANY-method} 
\alias{t,matrix.csr-method} 
\alias{t,matrix.csc-method} 
\alias{t,matrix.coo-method} 
\alias{diag,ANY-method} 
\alias{diag,matrix.csr-method} 
\alias{diag<-,ANY-method} 
\alias{diag<-,matrix.csr-method} 
\alias{diag<-,matrix.diag.csr-method} 
\alias{diff,matrix.csr-method} 
\alias{diff<-,ANY-method} 
\alias{diff<-,matrix.csr-method} 
\alias{diag.assign,matrix.csr-method} 
\alias{ncol,matrix.csr-method} 
\alias{nrow,matrix.csr-method} 
\alias{dim,ANY-method} 
\alias{dim,matrix.csr-method} 
\alias{dim,matrix.csc-method} 
\alias{dim,matrix.ssr-method} 
\alias{dim,matrix.ssc-method} 
\alias{dim,matrix.coo-method} 
\alias{rbind.matrix.csr} 
\alias{cbind.matrix.csr} 
\alias{[.matrix.csr}
\alias{[.matrix.diag.csr}
\alias{[<-.matrix.csr}
\alias{[.matrix.coo}
\alias{[<-.matrix.coo}
\title{Basic Linear Algebra for Sparse Matrices}
\description{Basic linear algebra operations for sparse matrices
of class \code{matrix.csr}.  
}
\usage{ }
\arguments{
\item{x}{matrix of class \code{matrix.csr}.}
\item{y}{matrix of class \code{matrix.csr} or a dense matrix or vector.}
\item{value}{replacement values.}
\item{i,j}{vectors of elements to extract or replace.}
\item{nrow}{optional number of rows for the result.}
\item{lag}{an integer indicating which lag to use.}
\item{differences}{an integer indicating the order of the difference.}
}
\details{Linear algebra operations for matrices of class 
\code{matrix.csr} are designed to behave exactly as for 
regular matrices.  In particular, matrix multiplication, kronecker
product, addition, 
subtraction and various logical operations should work as with the conventional
dense form of matrix storage, as does indexing, rbind, cbind, and diagonal
assignment and extraction.  The method diag may be used to extract the
diagonal of a \code{matrix.csr} object, to create a sparse diagonal see
\code{SparseM.ontology}.

The function \code{det} computes the determinant of the argument
matrix.  If the matrix is of class \code{matrix.csr} then it must
be symmetric, or an error will be returned.  If the matrix is of
class \code{matrix.csr.chol} then the determinant of the Cholesky
factor is returned, ie the product of the diagonal elements.

The function \code{norm} is used to check for symmetry by 
computing the maximum of the elements of the difference between
the matrix and its transpose.  Optionally, this sup norm can
be replaced by the Hilbert-Schmidt norm, or the l1 norm.
}
\references{ 
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R,\cr
\url{http://www.econ.uiuc.edu/~roger/research}
}
\seealso{
\code{slm} for sparse linear model fitting.  
\code{SparseM.ontology} for coercion and other class relations involving the
sparse matrix classes.
}
\examples{
n1 <- 10
n2 <- 10
p <- 6
y <- rnorm(n1)
a <- rnorm(n1*p)
a[abs(a)<0.5] <- 0
A <- matrix(a,n1,p)
A.csr <- as.matrix.csr(A)
b <- rnorm(n2*p)
b[abs(b)<1.0] <- 0
B <- matrix(b,n2,p)
B.csr <- as.matrix.csr(B)

# matrix transposition and multiplication
A.csr\%*\%t(B.csr)

# kronecker product
A.csr \%x\% matrix(1:4,2,2)

}

\keyword{algebra}

\eof
\name{SparseM.solve}
\alias{SparseM.solve}
\alias{chol,ANY-method}
\alias{chol,matrix.csr-method}
\alias{chol,matrix.csc-method}              
\alias{chol,matrix-method}              
\alias{chol}              
\alias{chol.default}              
\alias{backsolve-methods}
\alias{backsolve,ANY-method}
\alias{backsolve}
\alias{solve}
\alias{backsolve,matrix.csr.chol-method}
\alias{solve,ANY-method}
\alias{solve,matrix.csr-method}
\title{Linear Equation Solving for Sparse Matrices}
\description{
\code{chol} performs a Cholesky
decomposition of a symmetric positive definite sparse matrix \code{x} 
of class \code{matrix.csr}. \cr
\code{backsolve} performs a triangular back-fitting to compute
the solutions of a system of linear equations.  \cr
\code{solve} 
combines \code{chol} and \code{backsolve} and will
compute the inverse of a matrix if the right-hand-side is missing.
}
\usage{
chol(x, \dots)
backsolve(r, x, k, upper.tri, transpose)
solve(a, b, \dots)
}
\arguments{
\item{a}{symmetric positive definite matrix of class \code{matrix.csr}.}
\item{r}{object of class \code{matrix.csr.chol} returned by the function
\code{chol}.}
\item{x,b}{vector(regular matrix) of right-hand-side(s) of a system of linear equations.}
\item{k}{inherited from the generic; not used here.}
#\item{pivot}{inherited from the generic; not used here.}
#\item{nsubmax,nnzlmax,tmpmax}{dimensions of work arrays, in normal operation these are determined inside the algorithm.}
\item{upper.tri}{inherited from the generic; not used here.}
\item{transpose}{inherited from the generic; not used here.}
\item{\dots}{further arguments passed to or from other methods.}
}
\details{
\code{chol} performs a Cholesky decomposition of
a symmetric positive definite sparse matrix \code{x} of class 
\code{matrix.csr} using the block sparse Cholesky algorithm of Ng and 
Peyton (1993).
\code{backsolve} does triangular back-fitting to compute
the solutions of a system of linear equations.  For systems of linear equations
that only vary on the right-hand-side, the result from \code{chol}
can be reused.
\code{solve} 
combines \code{chol} and \code{backsolve}, and will
compute the inverse of a matrix if the right-hand-side is missing.
The determinant of the Cholesky factor is returned providing a
means to efficiently compute the determinant of sparse positive
definite symmetric matrices.
}
\references{ 
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R,\cr
\url{http://www.econ.uiuc.edu/~roger/research}

Ng, E. G. and B. W. Peyton (1993), "Block sparse Cholesky algorithms
on advanced uniprocessor computers", \emph{SIAM J. Sci. Comput.}, \bold{14},
pp. 1034-1056.

}
\seealso{
\code{slm} for sparse version of \code{lm} \cr
}
\examples{
data(lsq)
class(lsq) # -> [1] "matrix.csc.hb"
model.matrix(lsq)->design.o
class(design.o) # -> "matrix.csr"
dim(design.o) # -> [1] 1850  712
y <- model.response(lsq) # extract the rhs
length(y) # [1] 1850
t(design.o)\%*\%design.o -> XpX
t(design.o)\%*\%y -> Xpy
chol(XpX)->chol.o
backsolve(chol.o,Xpy)-> b1 # least squares solutions in two steps
solve(XpX,Xpy) -> b2 # least squares estimates in one step
}

\keyword{algebra}

\eof
\name{character or NULL-class}
\docType{class}
\alias{character or NULL-class}
\title{Class "character or NULL"}
\description{A virtual class needed by the "matrix.csc.hb" class}
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}


\section{Methods}{
No methods defined with class "character or NULL" in the signature.
}
\keyword{classes}


\eof
\name{lsq}
\alias{lsq}
\title{Least Squares Problems in Surveying}
\description{One of the four matrices from the least-squares solution of 
problems in surveying that were used by Michael Saunders and Chris Paige 
in the testing of LSQR}
\usage{data(lsq)}
\format{
A list of class \code{matrix.csc.hb} or \code{matrix.ssc.hb} depending
on how the coefficient matrix is stored with the following components:
\itemize{
  \item{ra }{ra component of the csc or ssc format of the coefficient matrix, X.
}   \item{ja }{ja component of the csc or ssc format of the coefficient matrix, X.
}   \item{ia }{ia component of the csc or ssc format of the coefficient matrix, X.
}   \item{rhs.ra }{ra component of the right-hand-side, y, if stored in csc or
        ssc format; right-hand-side stored in dense vector or matrix otherwise.}
  \item{rhs.ja }{ja component of the right-hand-side, y, if stored in csc or 
        ssc format; a null vector otherwise.}
  \item{rhs.ia }{ia component of the right-hand-side, y, if stored in csc or
        ssc format; a null vector otherwise.}
  \item{xexact}{vector of the exact solutions, b, if they exist; a null vector o
therwise.}  \item{guess}{vector of the initial guess of the solutions if they exist;
        a null vector otherwise.}
  \item{dim}{dimenson of the coefficient matrix, X.}
  \item{rhs.dim}{dimenson of the right-hand-side, y.}
  \item{rhs.mode}{storage mode of the right-hand-side; can be full storage or
        same format as the coefficient matrix.}
}
}
\seealso{\code{read.matrix.hb}, \code{write.matrix.hb}}
\examples{
data(lsq)
class(lsq) # -> [1] "matrix.csc.hb"
model.matrix(lsq)->X
class(X) # -> "matrix.csr"
dim(X) # -> [1] 1850  712
y <- model.response(lsq) # extract the rhs
length(y) # [1] 1850 
}
\references{
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R,\cr
\url{http://www.econ.uiuc.edu/~roger/research}

Matrix Market, \url{http://math.nist.gov/MatrixMarket/data/Harwell-Boeing/lsq/lsq.html}
}
\keyword{datasets}

\eof
\name{matrix.coo-class}
\docType{class}
\alias{matrix.coo-class}
\title{Class "matrix.coo"}
\description{A new class for sparse matrices stored in coordinate format}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("matrix.coo", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{ra}:}{Object of class \code{numeric}, a real array of nnz elements containing the non-zero
        elements of A.} 
    \item{\code{ja}:}{Object of class \code{integer}, an integer array of nnz elements containing the column
        indices of the elements stored in `ra'.}
    \item{\code{ia}:}{Object of class \code{integer}, an integer array of nnz elements containing the row
        indices of the elements stored in `ra'.}
    \item{\code{dimension}:}{Object of class \code{integer}, dimension of the matrix }
  }
}
\section{Methods}{
  \describe{
    \item{as.matrix.csr}{\code{signature(x = "matrix.coo")}: ... }
    \item{as.matrix}{\code{signature(x = "matrix.coo")}: ... }
    \item{dim}{\code{signature(x = "matrix.coo")}: ... }
  }
}

\seealso{ \code{\link{matrix.csr-class}}}

\keyword{classes}


\eof
\name{matrix.csc-class}
\docType{class}
\alias{matrix.csc-class}
\title{Class "matrix.csc"}
\description{A new class for sparse matrices stored in compressed sparse column format}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("matrix.csc", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{ra}:}{Object of class \code{numeric}, a real array of nnz elements containing the non-zero
        elements of A, stored in column order. Thus, if i<j, all elements
        of column i precede elements from column j. The order of elements
        within the column is immaterial.}
    \item{\code{ja}:}{Object of class \code{integer}, an integer array of nnz elements containing the row
        indices of the elements stored in `ra'.}
    \item{\code{ia}:}{Object of class \code{integer}, an integer array of n+1 elements containing pointers to
        the beginning of each column in the arrays `ra' and `ja'. Thus
        `ia[i]' indicates the position in the arrays `ra' and `ja'
        where the ith column begins. The last, (n+1)st, element of `ia'
        indicates where the n+1 column would start, if it existed.
 }
    \item{\code{dimension}:}{Object of class \code{integer}, dimension of the matrix }
  }
}
\section{Methods}{
  \describe{
    \item{as.matrix.csr}{\code{signature(x = "matrix.csc")}: ... }
    \item{as.matrix.ssc}{\code{signature(x = "matrix.csc")}: ... }
    \item{as.matrix.ssr}{\code{signature(x = "matrix.csc")}: ... }
    \item{as.matrix}{\code{signature(x = "matrix.csc")}: ... }
    \item{chol}{\code{signature(x = "matrix.csc")}: ... }
    \item{dim}{\code{signature(x = "matrix.csc")}: ... }
    \item{t}{\code{signature(x = "matrix.csc")}: ... }
  }
}

\seealso{ \code{\link{matrix.csr-class}}}

\keyword{classes}


\eof
\name{matrix.csc.hb-class}
\docType{class}
\alias{matrix.csc.hb-class}
\title{Class "matrix.csc.hb"}
\description{A new class consists of the coefficient matrix and
the right-hand-side of a linear system of equations,
initial guess of the solution and the exact solutions if they exist stored
in external files using the Harwell-Boeing format.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("matrix.csc.hb", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{ra}:}{Object of class \code{numeric}, ra component of the csc or ssc format of the coefficient matrix, X.}
    \item{\code{ja}:}{Object of class \code{integer}, ja component of the csc or ssc format of the coefficient matrix, X.}
    \item{\code{ia}:}{Object of class \code{numeric}, ia component of the csc or ssc format of the coefficient matrix, X.}
    \item{\code{rhs.ra}:}{Object of class \code{numeric}, ra component of the right-hand-side, y, if stored in csc or
        ssc format; right-hand-side stored in dense vector or matrix otherwise.}
    \item{\code{guess}:}{Object of class \code{numeric} or \code{NULL} vector of the initial guess of the solutions if they exist;
        a null vector otherwise.}
    \item{\code{xexact}:}{Object of class \code{numeric or NULL} vector of the exact solutions, b, if they exist; a null vector otherwise.}
    \item{\code{dimension}:}{Object of class \code{integer}, dimenson of the coefficient matrix, X.}
    \item{\code{rhs.dim}:}{Object of class \code{integer}, dimenson of the right-hand-side, y.}
    \item{\code{rhs.mode}:}{Object of class \code{character or NULL} storage mode of the right-hand-side; can be full storage or
        same format as the coefficient matrix.}
  }
}

\section{Methods}{
  \describe{
    \item{model.matrix}{\code{signature(object = "matrix.csc.hb")}: ... }
  }
}
\seealso{\code{\link{model.matrix}}, \code{\link{model.response}}, 
\code{\link{read.matrix.hb}}, \code{\link{write.matrix.hb}}, \code{\link{matrix.ssc.hb-class}}}
\keyword{classes}


\eof
\name{matrix.csr-class}
\docType{class}
\alias{matrix.csr-class}
\alias{matrix.diag.csr-class}
\title{Class "matrix.csr"}
\description{A new class for sparse matrices stored in compressed sparse row format}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("matrix.csr", ...)}.
and coerced from various other formats.  Coercion of integer scalars 
and vectors into identity matrices and diagonal matrices respectively
is accomplished by \code{as(x,"matrix.diag.csr")} which generates an
object that has all the rights and responsibilties of the matrix.csr class.
The default matrix.csr object is a scalar (1 by 1) matrix  with element 0.
}
\section{Slots}{
  \describe{
    \item{\code{ra}:}{Object of class \code{numeric}, a real array of nnz elements containing the non-zero
        elements of A, stored in row order. Thus, if i<j, all elements
        of row i precede elements from row j. The order of elements
        within the rows is immaterial.}
    \item{\code{ja}:}{Object of class \code{integer}, an integer array of nnz elements containing the column
        indices of the elements stored in `ra'.}
    \item{\code{ia}:}{Object of class \code{integer}, an integer array of n+1 elements containing pointers to
        the beginning of each row in the arrays `ra' and `ja'. Thus
        `ia[i]' indicates the position in the arrays `ra' and `ja'
        where the ith row begins. The last, (n+1)st, element of `ia'
        indicates where the n+1 row would start, if it existed.
 }
    \item{\code{dimension}:}{Object of class \code{integer}, dimension of the matrix }
  }
}

\section{Methods}{
  \describe{
    \item{\%*\%}{\code{signature(x = "matrix.csr", y = "matrix.csr")}: ... }
    \item{\%*\%}{\code{signature(x = "matrix.csr", y = "matrix")}: ... }
    \item{\%*\%}{\code{signature(x = "matrix.csr", y = "numeric")}: ... }
    \item{\%*\%}{\code{signature(x = "matrix", y = "matrix.csr")}: ... }
    \item{\%*\%}{\code{signature(x = "numeric", y = "matrix.csr")}: ... }
    \item{as.matrix.csc}{\code{signature(x = "matrix.csr")}: ... }
    \item{as.matrix.ssc}{\code{signature(x = "matrix.csr")}: ... }
    \item{as.matrix.ssr}{\code{signature(x = "matrix.csr")}: ... }
    \item{as.matrix.coo}{\code{signature(x = "matrix.csr")}: ... }
    \item{as.matrix}{\code{signature(x = "matrix.csr")}: ... }
    \item{chol}{\code{signature(x = "matrix.csr")}: ... }
    \item{diag}{\code{signature(x = "matrix.csr")}: ... }
    \item{diag<-}{\code{signature(x = "matrix.csr")}: ... }
    \item{dim}{\code{signature(x = "matrix.csr")}: ... }
    \item{image}{\code{signature(x = "matrix.csr")}: ... }
    \item{solve}{\code{signature(a = "matrix.csr")}: ... }
    \item{t}{\code{signature(x = "matrix.csr")}: ... }
    \item{diff}{\code{signature(x = "matrix.csr")}: ... }
  }
}
\seealso{\code{\link{matrix.csc-class}}}
\keyword{classes}


\eof
\name{matrix.csr.chol-class}
\docType{class}
\alias{matrix.csr.chol-class}
\title{Class "matrix.csr.chol"}
\description{A class of objects returned from Ng and Peyton's (1993) block
sparse Cholesky algorithm}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("matrix.csr.chol", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{nrow}:}{Object of class \code{integer}, number of rows in the linear system of equations }
    \item{\code{nnzlindx}:}{Object of class \code{numeric}, number of non-zero elements in lindx}
    \item{\code{nsuper}:}{Object of class \code{integer}, number of supernodes}
    \item{\code{lindx}:}{Object of class \code{integer}, vector of integer
containing, in column major order, the row subscripts of the non-zero entries 
in the Cholesky factor in a compressed storage format}
    \item{\code{xlindx}:}{Object of class \code{integer}, vector of integer of pointers for lindx}
    \item{\code{nnzl}:}{Object of class \code{numeric}, number of non-zero
entries, including the diagonal entries, of the Cholesky factor stored in lnz}
    \item{\code{lnz}:}{Object of class \code{numeric}, contains the entries of the Cholesky factor }
    \item{\code{xlnz}:}{Object of class \code{integer}, column pointer for the Cholesky factor stored in lnz}
    \item{\code{invp}:}{Object of class \code{integer}, vector of integer of inverse permutation vector}
    \item{\code{perm}:}{Object of class \code{integer}, vector of integer of permutation vector }
    \item{\code{xsuper}:}{Object of class \code{integer}, array containing the supernode partioning}
    \item{\code{det}:}{Object of class \code{numeric}, determinant of the Cholesky factor}
    \item{\code{ierr}:}{Object of class \code{integer}, error flag }
    \item{\code{time}:}{Object of class \code{numeric} execution time }
  }
}

\section{Methods}{
  \describe{
    \item{backsolve}{\code{signature(r = "matrix.csr.chol")}: ... }
  }
}
\seealso{\code{\link{chol}}, \code{\link{backsolve}}}
\keyword{classes}


\eof
\name{matrix.ssc-class}
\docType{class}
\alias{matrix.ssc-class}
\title{Class "matrix.ssc"}
\description{A new class for sparse matrices stored in symmetric sparse column format}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("matrix.ssc", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{ra}:}{Object of class \code{numeric}, a real array of nnz elements containing the non-zero
        elements of the lower triangular part of A, stored in column order. Thus, if i<j, all elements
        of column i precede elements from column j. The order of elements
        within the column is immaterial.}
    \item{\code{ja}:}{Object of class \code{integer}, an integer array of nnz elements containing the row
        indices of the elements stored in `ra'.}
    \item{\code{ia}:}{Object of class \code{integer}, an integer array of n+1 elements containing pointers to
        the beginning of each column in the arrays `ra' and `ja'. Thus
        `ia[i]' indicates the position in the arrays `ra' and `ja'
        where the ith column begins. The last, (n+1)st, element of `ia'
        indicates where the n+1 column would start, if it existed.
 }
    \item{\code{dimension}:}{Object of class \code{integer}, dimension of the matrix }
  }
}
\section{Methods}{
  \describe{
    \item{as.matrix.csc}{\code{signature(x = "matrix.ssc")}: ... }
    \item{as.matrix.csr}{\code{signature(x = "matrix.ssc")}: ... }
    \item{as.matrix.ssr}{\code{signature(x = "matrix.ssc")}: ... }
    \item{as.matrix}{\code{signature(x = "matrix.ssc")}: ... }
    \item{dim}{\code{signature(x = "matrix.ssc")}: ... }
  }
}
\seealso{\code{\link{matrix.csr-class}}}
\keyword{classes}


\eof
\name{matrix.ssc.hb-class}
\docType{class}
\alias{matrix.ssc.hb-class}
\title{Class "matrix.ssc.hb"}
\description{A new class consists of the coefficient matrix and
the right-hand-side of a linear system of equations,
initial guess of the solution and the exact solutions if they exist stored
in external files using the Harwell-Boeing format.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("matrix.ssc.hb", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{ra}:}{Object of class \code{numeric}, ra component of the csc or ssc format of the coefficient matrix, X.}
    \item{\code{ja}:}{Object of class \code{integer}, ja component of the csc or s
sc format of the coefficient matrix, X.}
    \item{\code{ia}:}{Object of class \code{integer}, ia component of the csc or ssc format of the coefficient matrix, X.}
    \item{\code{rhs.ra}:}{Object of class \code{numeric}, ra component of the right-hand-side, y, if stored in csc or
        ssc format; right-hand-side stored in dense vector or matrix otherwise.}
    \item{\code{guess}:}{Object of class \code{numeric or NULL} vector of the initial guess of the solutions if they exist;
        a null vector otherwise.}
    \item{\code{xexact}:}{Object of class \code{numeric or NULL} vector of the exact solutions, b, if they exist; a null vector otherwise.}
    \item{\code{dimension}:}{Object of class \code{integer}, dimenson of the coefficient matrix, X.}
    \item{\code{rhs.dim}:}{Object of class \code{integer}, dimenson of the right-hand-side, y.}
    \item{\code{rhs.mode}:}{Object of class \code{character or NULL} storage mode of the right-hand-side; can be full storage or
        same format as the coefficient matrix.}
  }
}
\section{Extends}{
Class \code{"matrix.csc.hb"}, directly.
}
\section{Methods}{
  \describe{
    \item{model.matrix}{\code{signature(object = "matrix.ssc.hb")}: ... }
  }
}
\seealso{\code{\link{model.matrix}}, \code{\link{model.response}}, 
\code{\link{read.matrix.hb}}, \code{\link{write.matrix.hb}}, \code{\link{matrix.csc.hb-class}}}
\keyword{classes}


\eof
\name{matrix.ssr-class}
\docType{class}
\alias{matrix.ssr-class}
\title{Class "matrix.ssr"}
\description{A new class for sparse matrices stored in symmetric sparse row format}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("matrix.ssr", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{ra}:}{Object of class \code{numeric}, a real array of nnz elements containing the non-zero
        elements of the lower triangular part of A, stored in row order. Thus, if i<j, all elements
        of row i precede elements from row j. The order of elements
        within the rows is immaterial.}
    \item{\code{ja}:}{Object of class \code{integer}, an integer array of nnz elements containing the column
        indices of the elements stored in `ra'.}
    \item{\code{ia}:}{Object of class \code{integer}, an integer array of n+1 elements containing pointers to
        the beginning of each row in the arrays `ra' and `ja'. Thus
        `ia[i]' indicates the position in the arrays `ra' and `ja'
        where the ith row begins. The last, (n+1)st, element of `ia'
        indicates where the n+1 row would start, if it existed.
 }
    \item{\code{dimension}:}{Object of class \code{integer}, dimension of the matrix }
  }
}
\section{Methods}{
  \describe{
    \item{as.matrix.csc}{\code{signature(x = "matrix.ssr")}: ... }
    \item{as.matrix.csr}{\code{signature(x = "matrix.ssr")}: ... }
    \item{as.matrix.ssr}{\code{signature(x = "matrix.ssr")}: ... }
    \item{as.matrix}{\code{signature(x = "matrix.ssr")}: ... }
    \item{dim}{\code{signature(x = "matrix.ssr")}: ... }
  }
}
\seealso{\code{\link{matrix.csr-class}}}
\keyword{classes}


\eof
\name{mslm-class}
\docType{class}
\alias{mslm-class}
\title{Class "mslm"}
\description{A sparse extension of \code{lm}}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("mslm", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{coefficients}:}{Object of class \code{numeric} estimated coefficients}
    \item{\code{chol}:}{Object of class \code{matrix.csr.chol} generated by 
the function \code{chol}}
    \item{\code{residuals}:}{Object of class \code{"numeric"} residuals}
    \item{\code{fitted}:}{Object of class \code{"numeric"} fitted values}
  }
}
\section{Extends}{
Class \code{"lm"}, directly.
Class \code{"slm"}, directly.
Class \code{"oldClass"}, by class "lm".
}
\section{Methods}{
  \describe{
    \item{coef}{\code{signature(object = "mslm")}: ... }
    \item{fitted}{\code{signature(object = "mslm")}: ... }
    \item{residuals}{\code{signature(object = "mslm")}: ... }
    \item{summary}{\code{signature(object = "mslm")}: ... }
  }
}
\seealso{ \code{\link{slm}}}
\keyword{classes}


\eof
\name{numeric or NULL-class}
\docType{class}
\alias{numeric or NULL-class}
\title{Class "numeric or NULL"}
\description{A virtual class needed by the "matrix.csc.hb" class}
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}


\section{Methods}{
No methods defined with class "numeric or NULL" in the signature.
}
\keyword{classes}


\eof
\name{slm-class}
\docType{class}
\alias{slm-class}
\title{Class "slm"}
\description{A sparse extension of \code{lm}}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("slm", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{coefficients}:}{Object of class \code{numeric} estimated coefficients}
    \item{\code{chol}:}{Object of class \code{matrix.csr.chol} generated by
function \code{chol}}
    \item{\code{residuals}:}{Object of class \code{"numeric"} residuals}
    \item{\code{fitted}:}{Object of class \code{"numeric"} fitted values}
  }
}
\section{Extends}{
Class \code{"lm"}, directly.
Class \code{"oldClass"}, by class "lm".
}
\section{Methods}{
  \describe{
    \item{coef}{\code{signature(object = "slm")}: ... }
    \item{fitted}{\code{signature(object = "slm")}: ... }
    \item{residuals}{\code{signature(object = "slm")}: ... }
    \item{summary}{\code{signature(object = "slm")}: ... }
  }
}
\seealso{ \code{\link{slm}}}
\keyword{classes}


\eof
\name{slm}
\alias{slm}
\title{Fit a linear regression model using sparse matrix algebra}
\description{
This is a function to illustrate the use of sparse linear algebra
to solve a linear least squares problem using Cholesky decomposition.
The syntax and output attempt to emulate \code{lm()} but may
fail to do so fully satisfactorily.  Ideally, this would eventually
become a method for \code{lm}.  The main obstacle to this step is
that it would be necessary to have a model.matrix function that
returned an object in sparse csr form.  For the present, the objects
represented in the formula must be in dense form.  If the user wishes
to specify fitting with a design matrix that is already in sparse form,
then the lower level function \code{slm.fit()} should be used. 
}
\usage{
slm(formula, data, weights, na.action, method = "csr", contrasts = NULL, ...)
}
\arguments{
\item{formula}{
    a formula object, with the response on the left of a \code{~} operator,
    and the terms, separated by \code{+} operators, on the right.  As in
    \code{lm()}, the response variable in the formula can be matrix valued.
  }
  \item{data}{
    a data.frame in which to interpret the variables
    named in the formula, or in the subset and the weights argument.
    If this is missing, then the variables in the formula should be on the
    search list.  This may also be a single number to handle some special
    cases -- see below for details.
  }
\item{weights}{
    vector of observation weights; if supplied, the algorithm fits
    to minimize the sum of the weights multiplied into the
    absolute residuals. The length of weights must be the same as
    the number of observations.  The weights must be nonnegative
    and it is strongly recommended that they be strictly positive,
    since zero weights are ambiguous.
  }
  \item{na.action}{
    a function to filter missing data.
    This is applied to the model.frame after any subset argument has been used.
    The default (with \code{na.fail}) is to create an error if any missing 
    values are found.  A possible alternative is \code{na.omit}, which
    deletes observations that contain one or more missing values.
  }
\item{method}{there is only one method based on Cholesky factorization}
\item{contrasts}{
    a list giving contrasts for some or all of the factors
    default = \code{NULL} appearing in the model formula.
    The elements of the list should have the same name as the variable
    and should be either a contrast matrix (specifically, any full-rank
    matrix with as many rows as there are levels in the factor),
    or else a function to compute such a matrix given the number of levels.
  }
\item{...}{
    additional arguments for the fitting routines
  }

}
}
\value{
A list of class \code{slm} consisting of:
  \item{coefficients}{estimated coefficients}
  \item{chol}{cholesky object from fitting}
  \item{residuals}{residuals}
  \item{fitted}{fitted values}
  \item{terms}{terms}
  \item{call}{call}
  ...
}
\references{
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R, \cr
\url{http://www.econ.uiuc.edu/~roger/research}
}
\author{ Roger Koenker }

\seealso{
\code{slm.methods} for methods \code{summary}, \code{print}, \code{fitted},
\code{residuals} and \code{coef} associated with class \code{slm},
and \code{slm.fit} for lower level fitting functions.  The latter functions
are of special interest if you would like to pass a sparse form of the
design matrix directly to the fitting process.}
\examples{
data(lsq)
X <- model.matrix(lsq) #extract the design matrix
y <- model.response(lsq) # extract the rhs
X1 <- as.matrix(X)
slm.time <- unix.time(slm(y~X1-1) -> slm.o) # pretty fast
lm.time <- unix.time(lm(y~X1-1) -> lm.o) # very slow
cat("slm time =",slm.time,"\n")
cat("slm Results: Reported Coefficients Truncated to 5  ","\n")
sum.slm <- summary(slm.o)
sum.slm$coef <- sum.slm$coef[1:5,]
sum.slm
cat("lm time =",lm.time,"\n")
cat("lm Results: Reported Coefficients Truncated to 5  ","\n")
sum.lm <- summary(lm.o)
sum.lm$coef <- sum.lm$coef[1:5,]
sum.lm
}

\keyword{regression}

\eof
\name{slm.fit}
\alias{slm.fit}
\alias{slm.fit}
\alias{slm.wfit}
\alias{slm.fit.csr}
\title{Internal slm fitting functions}
\description{
 Fitting functions for sparse linear model fitting.
}
\usage{
slm.fit(x,y,method, \dots)
slm.wfit(x,y,weights,\dots)
slm.fit.csr(x, y, \dots)
}
\arguments{
\item{x}{ design matrix.  }
  \item{y}{ vector of response observations.  }
  \item{method}{ only \code{csr} is supported currently }
  \item{weights}{an optional vector of weights to be used in the fitting
     process. If specified, weighted least squares is used with
     weights `weights' (that is, minimizing \deqn{\sum w_i*e_i^2}{sum w e^2}
    The length of weights must be the same as
    the number of observations.  The weights must be nonnegative
    and it is strongly recommended that they be strictly positive,
    since zero weights are ambiguous.}
  \item{\dots}{additional arguments.}
}
\details{
   \code{slm.fit} and \code{slm.wfit} call \code{slm.fit.csr} 
   to do Cholesky decomposition
   and then backsolve to obtain the least squares estimated coefficients.
   These functions can be called directly if the user is willing to
   specify the design matrix in \code{matrix.csr} form.  This is often
   advantageous in large problems to reduce memory requirements.
}
\value{
A list of class \code{slm} consisting of:
  \item{coef}{estimated coefficients}
  \item{chol}{cholesky object from fitting}
  \item{residuals}{residuals}
  \item{fitted}{fitted values}
  \item{terms}{terms}
  \item{call}{call}
  ...
}
\references{
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R,\cr
\url{http://www.econ.uiuc.edu/~roger/research}}
}
\author{ Roger Koenker }

\seealso{  \code{\link{slm}} }
\example{
data(lsq)
X <- model.matrix(lsq) #extract the design matrix
y <- model.response(lsq) # extract the rhs
class(X) # -> "matrix.csr"
class(y) # -> NULL
slm.fit(X,y)->slm.fit.o # this is much more efficient in memory usage than slm()
slm(y~as.matrix(X)-1) -> slm.o # this requires X to be transformed into dense mode
cat("Difference between `slm.fit' and `slm' estimated coefficients =",sum(abs(slm.fit.o$coef-slm.o$coef)),"\n")
}

\keyword{ regression }

\eof
\name{slm.methods}
\alias{slm.methods}
\alias{summary.slm}
\alias{summary.mslm}
\alias{print.summary.slm}
\alias{print.slm}
\alias{fitted.slm}
\alias{residuals.slm}
\alias{coef.slm}
\title{Methods for slm objects}
\description{
  Summarize, print, and extract objects from  \code{\link{slm}} objects.
}
\usage{
summary.slm(object, correlation, \dots)
summary.mslm(object, \dots)
print.slm(x, digits, \dots)
print.summary.slm(x, digits, symbolic.cor, signif.stars, \dots)
fitted.slm(object, \dots)
residuals.slm(object, \dots)
coef.slm(object, \dots)
}
\arguments{
\item{object,x}{ object of class \code{slm}.  }
\item{digits}{minimum number of significant digits to be used for most numbers.}
\item{symbolic.cor}{logical; if \code{TRUE}, the correlation of coefficients
      will be printed. The default is \code{FALSE}}
\item{signif.stars}{logical; if \code{TRUE}, P-values are additionally encoded
          visually as ``significance stars'' in order to help scanning
          of long coefficient tables.  It defaults to the
          `show.signif.stars' slot of `options'. }
\item{correlation}{logical; if \code{TRUE}, the correlation matrix of the
          estimated parameters is returned and printed.}
\item{\dots}{additional arguments passed to methods.}
}
\value{
  \code{print.slm} and \code{print.summary.slm} return invisibly.
  \code{fitted.slm}, \code{residuals.slm}, and \code{coef.slm} 
  return the corresponding components of the \code{slm} object.
}
\references{
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R,\cr
\url{http://www.econ.uiuc.edu/~roger/research}
}

\author{ Roger Koenker }

\seealso{  
\code{slm} 
}
\examples{
data(lsq)
X <- model.matrix(lsq) #extract the design matrix
y <- model.response(lsq) # extract the rhs
X1 <- as.matrix(X)
slm.time <- unix.time(slm(y~X1-1) -> slm.o) # pretty fast
cat("slm time =",slm.time,"\n")
cat("slm Results: Reported Coefficients Truncated to 5  ","\n")
sum.slm <- summary(slm.o)
sum.slm$coef <- sum.slm$coef[1:5,]
sum.slm
fitted(slm.o)[1:10]
residuals(slm.o)[1:10]
coef(slm.o)[1:10]
}

\keyword{ regression }

\eof
\name{summary.mslm-class}
\docType{class}
\alias{summary.mslm-class}
\title{Class "summary.mslm" }
\description{Sparse version of \code{summary.lm}}
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}


\section{Methods}{
  \describe{
    \item{print}{\code{signature(x = "summary.mslm")}: ... }
  }
}
\keyword{classes}

\eof
\name{summary.slm-class}
\docType{class}
\alias{summary.slm-class}
\title{Class "summary.slm" }
\description{Sparse version of \code{summary.lm}}
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}


\section{Methods}{
  \describe{
    \item{print}{\code{signature(x = "summary.slm")}: ... }
  }
}
\keyword{classes}

\eof
\name{triogramX}
\alias{triogramX}
\alias{X}
\title{A Design Matrix for a Triogram Problem}
\description{This is a design matrix arising from a bivariate smoothing
problem using penalized triogram fitting.  It is used in the SparseM vignette
to illustrate the use of the sparse matrix image function.}
\usage{data(triogramX)}
\format{A 375 by 100 matrix stored in compressed sparse row format}
\reference{
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R,\cr
\url{http://www.econ.uiuc.edu/~roger/research}
}
\keyword{datasets}

\eof
